#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netdb.h>
#include <signal.h>

int
open_socket(const char *node, const char *service)
{
  struct addrinfo hints, *res;
  int sockfd;

  memset(&hints, 0, sizeof(hints));
  hints.ai_family   = AF_INET;
  hints.ai_socktype = SOCK_DGRAM;
  hints.ai_flags    = AI_PASSIVE;
  if(getaddrinfo(NULL, "9999", &hints, &res) != 0) {
    return -1;
  }

  if((sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0) {
    return -1;
  }
  if(bind(sockfd, res->ai_addr, res->ai_addrlen) < 0) {
    return -1;
  }

  return sockfd;
}

size_t
split_cmd(char *buf, char **cmd, char *argv[], size_t max_argc)
{
  size_t argc, i, n;

  n = strlen(buf);
  argc = 0;
  for(i = 0; i < n; i++) {
    if(argc >= max_argc-1) break;
    argv[argc++] = &buf[i];
    while(buf[i] != ' ' && buf[i] != '\n' && i < n) {
      i++;
    }
    buf[i] = '\0';
  }
  argv[argc] = NULL;
  *cmd = argv[0];

  return argc;
}

int
exec_cmd(char *buf)
{
  int pid;
  int p[2];
  char *cmd, *argv[10];
  size_t i, argc;

  argc = split_cmd(buf, &cmd, argv, 10);

  if(pipe(p) < 0) {
    perror("(execve-test) failed to open pipe");
    return -1;
  }

  switch(pid = fork()) {
  case -1: /* Error */
    perror("(execve-test) fork failed");
    return -1;
  case 0:  /* Child */
    printf("(execve-test/child) execv: %s\n", cmd);
    for(i = 0; i < argc; i++) {
      printf("(execve-test/child)  argv[%zu] = %s\n", i, argv[i]);
    }
    fflush(stdout);

    close(1);
    dup(p[1]);
    close(p[0]);

    execv(cmd, argv);
    perror("(execve-test/child) execve failed");
    kill(getppid(), SIGINT);
    exit(1);
  default: /* Parent */
    close(p[1]);
    return p[0];
  }

  return -1;
}

int
main(int argc, char *argv[])
{
  FILE *fp;
  int child_fd;
  char buf[4096];
  socklen_t addrlen;
  struct sockaddr_storage addr;

  int sockfd = open_socket("localhost", "9999");
  if(sockfd < 0) {
    fprintf(stderr, "(execve-test) failed to open socket\n");
    return 1;
  }

  addrlen = sizeof(addr);
  if(recvfrom(sockfd, buf, sizeof(buf), 0, (struct sockaddr*)&addr, &addrlen) < 0) {
    fprintf(stderr, "(execve-test) recvfrom failed\n");
    return 1;
  }

  if((child_fd = exec_cmd(buf)) < 0) {
    fprintf(stderr, "(execve-test) failed to exec command: %s\n", buf);
    return 1;
  }
  fp = fdopen(child_fd, "r");

  while(fgets(buf, sizeof(buf), fp)) {
    sendto(sockfd, buf, strlen(buf)+1, 0, (struct sockaddr*)&addr, addrlen);
  }

  fclose(fp);

  return 0;
}

